/****************************************************************************
 **
 ** Copyright (C) 2008-2009 Grigory A. Mozhaev.  All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** AUTHOR: Grigory A. Mozhaev <zcrendel@mail.ru>
 **
 ****************************************************************************/
//! -*- coding: UTF-8 -*-
#define SOURCE_CODING "UTF-8"

extern "C" {
    #include <locale.h>
    #include <signal.h>
    #include <unistd.h>
    #include <errno.h>
    #include <stdio.h>
}

#include <QtCore>
#include <QtXml>

#include <CApplication>

QTranslator *CApplication::mpTranslator = 0;
QSettings   *CApplication::mpSettings   = 0;
CGlobals    *CApplication::mpGlobals    = 0;


CApplication::CApplication(int &aArgc, char **apArgv) :
QApplication(aArgc,apArgv) {
#if HAVE_LOCALE
    initLocale();
#endif // #if HAVE_LOCALE

    initSettings(initCommandArgs());
    initGlobals();
    initTranslation();

    Q_ASSERT_X(mpSettings != 0, "CApplication constructor", "Settings pointer is NULL");
    Q_ASSERT_X(mpGlobals  != 0, "CApplication constructor", "Globals pointer is NULL");
}

// Устанавливаем локализацию
void CApplication::initLocale() {
    ::setlocale(LC_ALL,"");
    ::setlocale(LC_NUMERIC,"POSIX");

    char *vpLang = ::getenv("LANG");
    if (vpLang != 0 && vpLang[0] != '\0') {
        QLocale vLocale(vpLang);
        QLocale::setDefault(vLocale);
    }

    QTextCodec *vpC = 0;

#ifdef Q_OS_LINUX
    char *vpNlCodeset = ::nl_langinfo(CODESET);
    if (vpNlCodeset!=0 && vpNlCodeset[0] != '\0') {
        vpC = QTextCodec::codecForName(vpNlCodeset);
        if (vpC) 
            QTextCodec::setCodecForLocale(vpC);
    }
#elif defined(Q_OS_WIN32)
    char *vpNlCodeset = "UTF-8";
    vpC = QTextCodec::codecForName(vpNlCodeset);
    if (vpC) 
        QTextCodec::setCodecForLocale(vpC);
#endif

    if (!vpC) {
        vpC = QTextCodec::codecForLocale();
    }

    if (vpC) {
        QTextCodec::setCodecForCStrings(vpC);
        QTextCodec::setCodecForTr(vpC);
    }
}

QString CApplication::initCommandArgs() {
    QStringList vArguments = arguments();

    QFileInfo vProgramInfo(vArguments.at(0));
    QCoreApplication::setApplicationName(vProgramInfo.baseName());

    QString vConfigFileName = QString::null;
    /*
    if (vArguments.indexOf("config") != -1) {
        QVariant vConfigPath = vArguments.indexOf("config"));
        if (vConfigPath.isValid() && (vConfigPath.type() == QVariant::String)) {
            vConfigFileName = QDir::toNativeSeparators(vConfigPath.toString());
        } else {
            QString vDefaultConfigFileName;
            vDefaultConfigFileName = QLatin1String("config.xml");
            vConfigFileName = QDir::toNativeSeparators(QString("%1/%2")
                           .arg(QDir::currentPath())
                           .arg(vDefaultConfigFileName));
        }
        }*/
    return vConfigFileName;
}

void CApplication::initSettings(const QString &aConfigFileName, QSettings::Format aFormat) {
    Q_UNUSED(aFormat);

    if (!aConfigFileName.isNull()) {
        QFileInfo vFile(aConfigFileName);
        if (vFile.isFile() && vFile.exists()) {
            mpSettings = new QSettings(aConfigFileName);
        }
    }

    if (!mpSettings)
        mpSettings = new QSettings(QLatin1String("MySoft"), QLatin1String("Start Runner"));
}

void CApplication::initGlobals() {
    mpGlobals = new CGlobals;
}

// устанавливаем файл перевода
void CApplication::initTranslation() {
    if ((mpSettings) && (mpSettings->contains(QLatin1String("language")))) {
        mpTranslator = new QTranslator();
        QString vLang = mpSettings->value("language").toString().toLower();
        if (vLang.compare("russian") == 0) {
            mpTranslator->load("lang_ru");
            installTranslator(mpTranslator);
        } else {
            qDebug() << "Run without any translation";
        }
    }
}

CApplication::~CApplication() {
    if (mpSettings) delete mpSettings;
    if (mpGlobals)  delete mpGlobals;

    if (mpTranslator) {
        delete mpTranslator;
        mpTranslator = NULL;
    }
}

QSettings *CApplication::settings() {
    return mpSettings;
}

CGlobals  *CApplication::globals() {
    return mpGlobals;
}

#ifdef HAVE_GUI
void CApplication::storeWidgetStatus (const QString &aName,
    QWidget *aWidget, const QMap<QString, QVariant> *aOptions) {
    if ((!aName.isNull()) && (aWidget)) {
        QString vPath = QString("Session/Widgets/%1").arg(aName);

        qDebug() << "storeWidget:" << vPath;

        mpSettings->setValue(vPath+"/windowFlags", QVariant(aWidget->windowFlags()));
        mpSettings->setValue(vPath+"/windowState", QVariant(aWidget->windowState()));
        mpSettings->setValue(vPath+"/geometry",    aWidget->geometry());
    }
}

void CApplication::restoreWidgetStatus (const QString &aName,
    QWidget *aWidget, const QMap<QString, QVariant> *aOptions) {

    if ((!aName.isNull()) && (aWidget)) {
        QString vPath = QString("Session/Widgets/%1").arg(aName);

        qDebug() << "restoreWidget:" << vPath;

        if (mpSettings->contains(vPath+"/windowFlags"))
          aWidget->setWindowFlags(Qt::WindowFlags(mpSettings->value(vPath+"/windowFlags").toInt()));

        if (mpSettings->contains(vPath+"/windowState"))
          aWidget->setWindowState(Qt::WindowStates(mpSettings->value(vPath+"/windowState").toInt()));

        if (mpSettings->contains(vPath+"/geometry"))
          aWidget->setGeometry(mpSettings->value(vPath+"/geometry").toRect());
    }
}
#endif // HAVE_GUI

