/****************************************************************************
 **
 ** Copyright (C) 2008-2009 Grigory A. Mozhaev.  All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** AUTHOR: Grigory A. Mozhaev <zcrendel@mail.ru>
 **
 ****************************************************************************/
//! -*- coding: UTF-8 -*-
#define SOURCE_CODING "UTF-8"

#include <CFsUtils>

#include "3rdparty/quazip/quazip.h"
#include "3rdparty/quazip/quazipfile.h"

QString    CFsUtils::mErrorMessage = QString("");

/**
 * Функция позволяет получить списки файлов и папок, которые
 * содержатся в aRootDir, в результате рекурсивного поиска.
 * 
 * @param aRootDir корневая директория. 
 *
 * @param aDirectories ссылка на список, который будет заполнен
 * именами найденых директорий.  
 *
 * @param aFilesссылка на список, который будет заполнен
 * именами найденых файлов.
 *
 * @param aStripRoot true - удалять из имен файлов и директорий
 * значение aRootDir.
 * 
 * @return зарезервировано
 */
int CFsUtils::getFilesDirsLists(const QString &aRootDir,
                                QStringList &aDirectories,
                                QStringList &aFiles,
                                bool aStripRoot) {
    QDir vDir(aRootDir);

    QString vPath  = aRootDir;
    int     vLevel = 0;

    QList<QStringList> vDirs;
    QList<int>     vIndex;
    QList<QString> vPaths;

    aDirectories.append("");

    vDirs.append(aDirectories);
    aDirectories.clear();
    vIndex.append(0);
    vPaths.append(vPath);

    while (vLevel > -1) {
        int i; bool vUp = false;
        for (i=vIndex.at(vLevel); i < vDirs.at(vLevel).size(); i++) {
            vPath = vPaths.at(vLevel)+vDirs.at(vLevel).at(i)+QLatin1String("/");
            aDirectories.append(vPath);
            // get files
            QDir vLocDir(vPath);
            QStringList vLocFiles = vLocDir.entryList(QDir::Files);
            for (int j=0; j < vLocFiles.size();j++)
                aFiles.append(vPath+vLocFiles.at(j));
            //

            QStringList vNewList = vLocDir.entryList(QDir::AllDirs | QDir::Hidden 
                                                     | QDir::NoDotAndDotDot);
            if (vNewList.size() > 0) {
                vIndex[vLevel]   = i+1;
                vLevel++;
                if (vLevel >= vDirs.size()) {
                    vDirs.append(vNewList);
                    vPaths.append(vPath);
                    vIndex.append(0);
                } else {
                    vDirs[vLevel]  = vNewList;
                    vPaths[vLevel] = vPath;
                    vIndex[vLevel] = 0;
                }
                vUp = true;
                break;
            }
        }
        if (vUp) continue;
        if (i >= vDirs.at(vLevel).size()) vLevel--;
    }

    aDirectories.removeFirst();

    if (aStripRoot) {
      for (int i=0; i < aDirectories.size();i++) {
          aDirectories[i].replace(QDir::toNativeSeparators(QString("%1/").arg(aRootDir)),
              QLatin1String(""));
      }

      for (int i=0; i < aFiles.size();i++) {
          aFiles[i].replace(QDir::toNativeSeparators(QString("%1/").arg(aRootDir)),
                            QLatin1String(""));
      }
    }

    return ECODE_SUCCESS;
}

int CFsUtils::extractZipFile(const QString &aFileName,
                             const QString &vWorkDir) {
    QDir vDir(vWorkDir);
    QString vFileName = QDir::toNativeSeparators(aFileName);
    QuaZip vArchive(vFileName);//.toLocal8Bit().data());

    if (!vArchive.open(QuaZip::mdUnzip)) {
        mErrorMessage = tr("Can't open archive \"%1\"").arg(vFileName);
        return ECODE_FILE_OPEN;
    }

    int vFilesCount = vArchive.getEntriesCount();
    if (vFilesCount < 0) { // archive is empty
        mErrorMessage = tr("Can't extract some files of directories from zip archive\n"
                           "\"%1\""
                           "Details: %1")
            .arg(vFileName)
            .arg(vArchive.getZipError());

        vArchive.close();
        return ECODE_EXTRACT;
    }

    int vRetVal = ECODE_SUCCESS;
    for (bool vMore=vArchive.goToFirstFile(); vMore; vMore=vArchive.goToNextFile()) {
        QuaZipFileInfo vFileInfo;
        if (vArchive.getCurrentFileInfo(&vFileInfo)) {
            if (vFileInfo.name.right(1).compare(QLatin1String("/")) == 0 || 
                (vFileInfo.externalAttr == 0x41ED0010)) {
                if (!vDir.mkpath(QString("%1/%2").arg(vWorkDir).arg(vFileInfo.name))) {
                    vRetVal = ECODE_DIR_CREATE;
                    break;
                }
            } else {
                QDir locWorkDir(vWorkDir);
                QString vFileDir = 
                    QFileInfo (QDir::convertSeparators(QString("%1/%2")
                                                       .arg(vWorkDir)
                                                       .arg(vFileInfo.name))).path();
                if (!locWorkDir.exists(vFileDir)) {
                    if (!locWorkDir.mkpath(vFileDir)) {
                        vRetVal = ECODE_DIR_CREATE;
                        break;
                    }
                }

                QuaZipFile vFile(&vArchive);
                if (vFile.open(QIODevice::ReadOnly))  {
                    QFile vExtractFile;
                    QString vPreFileName = 
                        QDir::toNativeSeparators(QString("%1/%2").arg(vWorkDir).arg(vFileInfo.name));
                    vExtractFile.setFileName(vPreFileName);
                    if (vExtractFile.open(QIODevice::WriteOnly |
                                          QIODevice::Truncate)) {
                        if (vExtractFile.write(vFile.readAll()) == -1) {
                            vRetVal = ECODE_FILE_WRITE;
                            break;
                        }
                        vExtractFile.close();
                    } else {
                        qDebug() << "can't write this file '" << vPreFileName << "'.";
                        vRetVal = ECODE_FILE_CREATE;
                        break;
                    }
                    vFile.close();
                }
            }
        }
    }

    if (vRetVal!=ECODE_SUCCESS) {
        QString vDetails;

        if(vArchive.getZipError()!=UNZ_OK) {
            vDetails = tr("Details: %1").arg(vArchive.getZipError());
            vRetVal  = ECODE_EXTRACT;
        }
        mErrorMessage = tr("Can't extract some files or directories "
                           "from zip archive\n ArchiveName:%1\n%2")
            .arg(vFileName)
            .arg(vDetails);

        return vRetVal;
    }
    
    vArchive.close();
    return ECODE_SUCCESS;
}


int CFsUtils::compressZipFile(const QString &aFileName,
                                     const QString &vWorkDir) {
    QStringList vDirectories;
    QStringList vFiles;

    int vRetVal = getFilesDirsLists(vWorkDir, vDirectories, vFiles, false);

    if (vRetVal!=ECODE_SUCCESS) {
        return vRetVal;
    }

    // compressing
    QuaZip vArchive(aFileName.toLocal8Bit().data());
    if (!vArchive.open(QuaZip::mdCreate)) {
        mErrorMessage = tr("Can't create archive %1.\n"
                           "Details: %2")
            .arg(aFileName)
            .arg(vArchive.getZipError());
        return ECODE_FILE_CREATE;
    }

    bool vErrorFlag = false;
    for (int i=0; i < vDirectories.size();i++) {
        QString vDirName = vDirectories.at(i);
        vDirName.replace(QString("%1/").arg(vWorkDir), QLatin1String(""));
        QuaZipFile vFile(&vArchive);
        if (!vFile.open(QIODevice::WriteOnly, 
                        QuaZipNewInfo(vDirName)))  {
            vFile.close();
            vErrorFlag = true;
            break;
        }
        vFile.close();
    }

    for (int i=0; i < vFiles.size();i++) {
        QString vFileName = vFiles.at(i);
        vFileName.replace(QString("%1/").arg(vWorkDir), QLatin1String(""));
        QuaZipFile vFile(&vArchive);
        if (!vFile.open(QIODevice::WriteOnly, 
                        QuaZipNewInfo(vFileName)))  {
            vErrorFlag = true;
            break;
        }

        QFile vContent(vFiles.at(i));
        if (vContent.open(QIODevice::ReadOnly)) {
            vFile.write(vContent.readAll());
            vContent.close();

            if(vFile.getZipError()!=UNZ_OK) {
                vErrorFlag = true;
                break;
            }
        } else {
            vErrorFlag = true;
            break;
        }

        vFile.close();
        if(vFile.getZipError()!=UNZ_OK) {
            vErrorFlag = true;
            break;
        }
    }

    if (vErrorFlag) {
        mErrorMessage = tr("Can't create zip archive, "
                           "failed to add some files (or directories).\n"
                           "Details: %1").arg(vArchive.getZipError());
        vArchive.close();
        return ECODE_COMPRESS;
    }

    vArchive.close();
    if(vArchive.getZipError()!=UNZ_OK) {
        mErrorMessage = tr("Error was happened while closing zip-archive file.\n"
                           "Details: %1").arg(vArchive.getZipError());
        return ECODE_COMPRESS;
    }

    return ECODE_SUCCESS;
}

int CFsUtils::removeDirectory(const QString &aDirName) {
    QFileInfo vDirInfo(aDirName);
    if (vDirInfo.exists()) {
        if (!vDirInfo.isDir()) {
            return ECODE_NAME_ISNT_DIR;
        }

        QDir vDir;
        if (!aDirName.isEmpty()) {
            // Очистка и удаление рабочей директории
            QStringList vFiles, vDirs;
            
            int vRetVal = getFilesDirsLists(aDirName, vDirs, vFiles);

            if (vRetVal!=ECODE_SUCCESS)
                return vRetVal;
            
            for (int i=(vFiles.size() - 1); i >= 0; i--) {
                if (!QFile::remove(vFiles.at(i))) {
                    return ECODE_FILE_REMOVE;
                }
            }
            
            for (int i=(vDirs.size() - 1); i >= 0; i--) {
                if (!vDir.rmdir(vDirs.at(i))) {
                    return ECODE_DIR_REMOVE;
                }
            }
            return (vDir.rmdir(aDirName)) ? ECODE_DIR_REMOVE : ECODE_SUCCESS;
        }
    }

    return ECODE_SUCCESS;
}

int CFsUtils::copyDirectory(const QString &aFromPath,
                            const QString &aToPath,
                            bool  aOnlyContent) {
    QStringList vFiles;
    QStringList vDirs;

    QFileInfo vFileFrom(aFromPath);
    QFileInfo vFileTo(aToPath);
    
    if (!vFileFrom.isDir() || (!vFileTo.isDir())) {
        // error!
        mErrorMessage = tr("Can't copy '%1' to '%2'."                           
                           "Some of these paths isn't a directory.");
        return ECODE_NAME_ISNT_DIR;
    }

    QString vFromPath = QDir::toNativeSeparators(aFromPath);
    QString vToPath   = QDir::toNativeSeparators(aToPath);

    getFilesDirsLists(vFromPath, vDirs, vFiles, true);

    if (vToPath.right(1).compare(QDir::toNativeSeparators(QLatin1String("/"))) != 0) {
        vToPath += QDir::toNativeSeparators(QLatin1String("/"));
    }

    if (vFromPath.right(1).compare(QDir::toNativeSeparators(QLatin1String("/"))) == 0) {
        vFromPath.resize(vFromPath.size() - 1);
    }

    QString vTopDir = vFromPath.section(QDir::toNativeSeparators(QLatin1String("/")), -1);
    vFromPath += QDir::toNativeSeparators(QLatin1String("/"));

    QDir vDir(vToPath);
    if (!aOnlyContent) {
        if (vDir.mkdir(vTopDir)) {
            mErrorMessage = tr("Can't create directory '%1'").arg(vTopDir);
            return ECODE_DIR_CREATE;
        }

        vToPath += vTopDir + QDir::toNativeSeparators(QLatin1String("/"));
        if (vDir.cd(vTopDir)) {
            mErrorMessage = tr("Can't change directory to '%1'").arg(vTopDir);
            return ECODE_WRONG_PATH;
        }
    }

    for (int i=0; i < vDirs.size(); i++) {
        if (!vDir.mkpath(vDirs.at(i))) {
            mErrorMessage = tr("Can't create directory '%1'").arg(vDirs.at(i));
            return ECODE_DIR_CREATE;
        }
    }
    
    for (int i=0; i < vFiles.size(); i++) {
        if (!QFile::copy(QDir::toNativeSeparators(vFromPath+vFiles.at(i)),
                         QDir::toNativeSeparators(vToPath+vFiles.at(i)))) {
            mErrorMessage = tr("Can't copy file.\n"
                               "source path:'%1'\n"
                               "destination path '%2'")
                .arg(QDir::toNativeSeparators(vFromPath+vFiles.at(i)))
                .arg(QDir::toNativeSeparators(vToPath+vFiles.at(i)));

            qDebug() << mErrorMessage << " ------";
            qDebug() << "FROM: " << vFromPath;
            qDebug() << "TO: " << vToPath;
            
            return ECODE_FILE_COPY;
        }
    }

    return ECODE_SUCCESS;
}
