/****************************************************************************
 **
 ** Copyright (C) 2008-2009 Grigory A. Mozhaev.  All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** AUTHOR: Grigory A. Mozhaev <zcrendel@mail.ru>
 **
 ****************************************************************************/
#include <QtGui>

#include "chighlighter.h"

CHighlighter::CHighlighter(QTextDocument *apParent)
    : QSyntaxHighlighter(apParent)
{
    SRule vRule;

    mKeyword.setForeground(Qt::darkBlue);
    mKeyword.setFontWeight(QFont::Bold);

    QStringList vKeywords;
    vKeywords << "\\bvar\\b" 
              << "\\bCTag\\b" 
              << "\\bCRow\\b" 
              << "\\bCSection\\b" 
              << "\\bCReport\\b" 
              << "\\bchar\\b" 
              << "\\bdouble\\b" 
              << "\\bint\\b";

    foreach (QString vPattern, vKeywords) {
        vRule.mPattern = QRegExp(vPattern);
        vRule.mFormat  = mKeyword;
        mRules.append(vRule);
    }

    mSingleLineComment.setForeground(Qt::red);
    vRule.mPattern = QRegExp("//[^\n]*");
    vRule.mFormat  = mSingleLineComment;
    mRules.append(vRule);

    mMultiLineComment.setForeground(Qt::red);

    mQuotation.setForeground(Qt::darkGreen);
    vRule.mPattern = QRegExp("\".*\"");
    vRule.mPattern.setMinimal(true);
    vRule.mFormat  = mQuotation;
    mRules.append(vRule);

    //mFunction.setFontItalic(true);

    QStringList vFunctions;
    vFunctions << "\\bvalue\\b"
               << "\\bsetValue\\b"
               << "\\btag\\b"
               << "\\bsection\\b"
               << "\\brow\\b";

    mFunction.setForeground(Qt::blue);

    foreach (QString vPattern, vFunctions) {
        vRule.mPattern = QRegExp(QString("\\b%1+(?=\\()").arg(vPattern));
        vRule.mFormat  = mFunction;
        mRules.append(vRule);
    }
    
    /*
    mFunction.setForeground(Qt::blue);
    vRule.mPattern = QRegExp("\\b[A-Za-z0-9_]+(?=\\()");
    vRule.mFormat  = mFunction;
    mRules.append(vRule);
    */
    
    mCommentStart = QRegExp("/\\*");
    mCommentEnd   = QRegExp("\\*/");
}

void CHighlighter::highlightBlock(const QString &aText)
{
    foreach (SRule vRule, mRules) {
        QRegExp vExpression(vRule.mPattern);
        int vIndex = aText.indexOf(vExpression);
        while (vIndex >= 0) {
            int vLength = vExpression.matchedLength();
            setFormat(vIndex, vLength, vRule.mFormat);
            vIndex = aText.indexOf(vExpression, vIndex + vLength);
        }
    }

    setCurrentBlockState(0);

    int vStartIndex = 0;
    if (previousBlockState() != 1)
        vStartIndex = aText.indexOf(mCommentStart);

    while (vStartIndex >= 0) {
        int vEndIndex = aText.indexOf(mCommentEnd, vStartIndex);
        int vCommentLength;
        if (vEndIndex == -1) {
            setCurrentBlockState(1);
            vCommentLength = aText.length() - vStartIndex;
        } else {
            vCommentLength = vEndIndex - vStartIndex
                + mCommentEnd.matchedLength();
        }
        setFormat(vStartIndex, vCommentLength, mMultiLineComment);
        vStartIndex = aText.indexOf(mCommentStart, vStartIndex + vCommentLength);
    }
}
