/****************************************************************************
 **
 ** Copyright (C) 2008-2009 Grigory A. Mozhaev.  All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** AUTHOR: Grigory A. Mozhaev <zcrendel@mail.ru>
 **
 ****************************************************************************/
#define SOURCE_CODING "UTF-8"

extern "C" {
#if defined(HAVE_LOCALE)
    #include <locale.h>
#endif // defined(HAVE_LOCALE)
    #include <signal.h>
    #include <unistd.h>
    #include <errno.h>
    #include <stdio.h>
    #include <fcntl.h>
#if defined (Q_OS_UNIX)
    #include <linux/cdrom.h>
    #include <sys/ioctl.h>
#endif
    #include <sys/types.h>
    #include <sys/stat.h>
    #include <unistd.h>
}

#include <QtCore>
#include <QEvent>
#include <QtXml>

#include <CMainWindow>


CMainWindow::CMainWindow( QWidget * apParent, Qt::WFlags aFlags)
    : QMainWindow(apParent, aFlags)
{
    createActions();
    createMenus();
    createToolbars();
    setupEditor();    

    CReportFactory::init(qApp->applicationDirPath());

    setCentralWidget(mpTextEdit);

    setGeometry(100, 100, 640, 480);
    setWindowState(Qt::WindowMaximized);
}

void CMainWindow::setupEditor() {
    QFont vFont;
    vFont.setFamily("Courier");
    vFont.setFixedPitch(true);
    vFont.setPointSize(15);

    mpTextEdit = new QTextEdit;
    mpTextEdit->setFont(vFont);
    mpHighlighter = new CHighlighter(mpTextEdit->document());

    // FIXME: вынести в config
    QFile vFile("template.js");
    if (vFile.open(QFile::ReadOnly | QFile::Text)) {
        mpTextEdit->setPlainText(vFile.readAll());
    }
}

CMainWindow::~CMainWindow() {    
}

void CMainWindow::addAction(QString aName, QString aTitle, 
                            QString aShortCut, QString aResource, 
                            QString aTip, const QObject *aReceiver, const char *aSlot) {

    QAction *vAction = new QAction(QIcon(aResource), aTitle, this);
    vAction->setShortcut(aShortCut);
    vAction->setStatusTip(aTip);

    mActions.insert(aName, vAction);
    connect(vAction, SIGNAL(triggered()), aReceiver, aSlot);
}

void CMainWindow::createActions() {
    addAction("quit", tr("В&ыход"),  "Ctrl+Q",
              ":/icons/exit.png", tr("Завершить выполнение примера"), this, SLOT(quit()));

    addAction("openTemplate", tr("Выбрать &шаблон"),  "Ctrl+O",
              ":/icons/fileopen.png", 
              tr("Выбрать файл шаблона для генерации отчета"),
              this, SLOT(openTemplate()));

    addAction("generateReport", tr("Создать &отчет"),  
              "Ctrl+S",
              ":/icons/docproperties.png", 
              tr("Создать отчет и запустить OpenOffice.org для просмотра"), 
              this, SLOT(saveReport()));
}

void CMainWindow::addMenuAction(QMenu *apMenu, const QString &aAction) {
  if (!apMenu)
    return ;

  if (mActions.contains(aAction)) {
    apMenu->addAction(mActions.value(aAction));
  } else {
    qDebug() << tr("Error(menu): Action %1 doesn't exist").arg(aAction);
    QAction *vpAction = new QAction(tr("Error: Action %1 doesn't exist").arg(aAction), this);
    apMenu->addAction(vpAction);
  }
}

void CMainWindow::addToolBarAction(QToolBar *apToolBar, const QString &aAction) {
    if (!apToolBar) return ;

    if (mActions.contains(aAction)) {
        apToolBar->addAction(mActions.value(aAction));
    } 
    else {
        qDebug() << tr("Error(toolbar): Action %1 doesn't exist").arg(aAction);
        QAction *vpAction = new QAction(tr("Error: Action %1 doesn't exist").arg(aAction), this);
        apToolBar->addAction(vpAction);
    }
}

void CMainWindow::createMenus() {
    QMenu *vpActions   = menuBar()->addMenu(tr("&Пример"));
    
    if (vpActions) {
        addMenuAction(vpActions, "openTemplate");
        addMenuAction(vpActions, "generateReport");
        vpActions->addSeparator();
        addMenuAction(vpActions, "quit");

        mMenus.insert("actions",  vpActions);
    }
    else {
       qDebug("Can't create application menu");
    }
}

void CMainWindow::createToolbars() {
    QToolBar *vpActions   = addToolBar(tr("Dialog"));
    addToolBarAction(vpActions, "quit");

    QToolBar *vpFile      = addToolBar(tr("File"));
    addToolBarAction(vpFile, "openTemplate");
    addToolBarAction(vpFile, "generateReport");
    
    mToolBars.insert("actions",  vpActions);  
    mToolBars.insert("file",     vpFile);
}

void CMainWindow::quit() {
    qApp->quit();
}

void CMainWindow::none() {
    QMessageBox::warning(this, 
                      tr("Function stub"),
                      tr("Action is not defined"),
                      QMessageBox::Ok, QMessageBox::Ok);
}


int CMainWindow::init() {
    return 0;
}


void CMainWindow::openTemplate() {
    qWarning("open template");
    QString vFileName =
        QFileDialog::getOpenFileName(this,
                                     tr("Choose location and name for file"),
                                     "","odt Files (*.odt)");
    if (vFileName.size() > 0) {
        CReportFactory::clearTemplates();
        CReportFactory::addTemplate("test", vFileName);
    }
}

void CMainWindow::saveReport() {
    // Проверяем существование шаблона   
    executeScript();
}

int CMainWindow::executeScript() {
    return CReport::generateReport(mpTextEdit->toPlainText(),
                                   QDir::toNativeSeparators(QDir::currentPath()+"/tmp"));
}
