/****************************************************************************
 **
 ** Copyright (C) 2008-2009 Grigory A. Mozhaev.  All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** AUTHOR: Grigory A. Mozhaev <zcrendel@mail.ru>
 **
 ****************************************************************************/
#ifndef _CMAINWINDOW_H_
#define _CMAINWINDOW_H_

#include <QMainWindow>
#include <QMenuBar>
#include <QSettings>

#include <QtCore>
#include <QtGui>
#include <QtSql>
#include <QtScript>

#include <CReportFactory>
#include <CReport>

#include "chighlighter.h"

//#include <COdfReport>

/**
 * Класс CMainWindow - главное окно приложения. Содержит основные
 * визуальные виджеты, главное меню программы и панели инструментов.
 */
class CMainWindow : public QMainWindow {
Q_OBJECT

public:
    CMainWindow(QWidget *apParent = 0,  Qt::WFlags aFlags = 0 );
    ~CMainWindow();

    int init();

public slots: 
    void quit();
    void none();

    void saveReport();
    void openTemplate();

private:
    void addAction(QString aName, QString aTitle, QString aShortCut,
                   QString aResource, QString aTip, const QObject *aReceiver, const char *aSlot);

    void addMenuAction(QMenu *apMenu, const QString &aAction);
    void addToolBarAction(QToolBar *apToolBar, const QString &aAction);

    void createActions();
    void createMenus();
    void createToolbars();
    void setupEditor();

    int executeScript();

    QMap<QString, QAction *> mActions;

    // Меню главного окна
    QMap<QString, QMenu *>   mMenus;

    // Панели инструментов
    QMap<QString, QToolBar *> mToolBars;

    QSettings  *mpConfig;
    QTextEdit  *mpTextEdit;
    CHighlighter *mpHighlighter;

};

#endif // _CMAINWINDOW_H_
