/****************************************************************************
 **
 ** Copyright (C) 2008-2009 Grigory A. Mozhaev.  All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** AUTHOR: Grigory A. Mozhaev <zcrendel@mail.ru>
 **
 ****************************************************************************/
#ifndef _CREPORT_H_
#define _CREPORT_H_

#include <QtCore>
#include <QtGui>
#include <QtXml>
#include <QtSql>
#include <QtScript>
#include <IReport>

class CTag;
class CRow;
class CSection;

typedef CTag* CTagObj; 
typedef CRow* CRowObj;
typedef CSection* CSectionObj;

class CReport : public QObject, public IReportData, public QScriptable {
Q_OBJECT
Q_INTERFACES(IReportData)

public:
    CReport(QObject *apParent = 0);
    ~CReport();
 
    static int generateReport(const QString &aScript,
                              const QString &aWorkDir);
    static void addReportClasses(QScriptEngine &aEngine);

    QMap<QString,QVariant> options() {
        return mOptions;
    }

    QString templatePath(const QString &aName);
    QString workDir();

public slots:
    CTag *tag(const QString &aName);
    CSection *section(const QString &aName = QString::null); 

    int generate(const QString &aReportType);
    void setOption(const QString &aName, const QVariant &aValue);

private:
    // CReport <--> SQLite
    QSqlTableModel mRows;
    QSqlTableModel mSections;

    QMap<QString,QVariant> mOptions;

    static QString mWorkDir;
    friend class CSection;
};

class CSection : public QObject, public QScriptable {
Q_OBJECT

public:
    CSection(QObject *apParent = 0);
    CSection(CReport *apParent, const QString &aName);
    ~CSection();

    static QScriptValue toScriptValue(QScriptEngine* aEngine, const CSectionObj& aSection)
    {
        CSection* nonConstSection = const_cast<CSection*>(aSection);
        return aEngine->newQObject(nonConstSection);
    };

    static void fromScriptValue(const QScriptValue& aScriptValue, CSectionObj& aSection) 
    {
        QObject* qobj = aScriptValue.toQObject();
        aSection = static_cast<CSection*>(qobj);
    };

    static uint mMaxId;

public slots:
    QString name();
    CRow *row(uint aRow = 0);
    int rowCount();
    QVariant min(const QString &aTagName);
    QVariant max(const QString &aTagName);
    QVariant avg(const QString &aTagName);
    QVariant sum(const QString &aTagName);

private:
    QVariant value(uint aRow, const QString& aTagName);
    void setValue(uint &aRow, const QString& aTagName,const QVariant &aTagValue);

    QString mName;
    uint mId;

    CReport *mpReport;

    friend class CReport;
    friend class CRow;
};

class CRow : public QObject, public QScriptable {
Q_OBJECT

public:
    CRow(QObject *apParent = 0);
    CRow(CSection *apParent, uint aRow = 0);
    ~CRow();

    static QScriptValue toScriptValue(QScriptEngine* aEngine, const CRowObj& aRow)
    {
        CRow* nonConstRow = const_cast<CRow*>(aRow);
        return aEngine->newQObject(nonConstRow);
    };

    static void fromScriptValue(const QScriptValue& aScriptValue, CRowObj& aRow) 
    {
        QObject* qobj = aScriptValue.toQObject();
        aRow = static_cast<CRow*>(qobj);
    };

public slots:
    CTag *tag(const QString& aName);

private:
    void setValue(const QString& aTagName, const QVariant &aTagValue);
    QVariant value(const QString& aTagName);

    uint mRow;
    CSection    *mpSection;

    friend class CSection;
    friend class CTag;
};

class CTag : public QObject, public QScriptable {
Q_OBJECT

public:
    CTag(QObject *apParent = 0);
    CTag(CRow *apParent, const QString &aName);
    ~CTag();

    static QScriptValue toScriptValue(QScriptEngine* aEngine, const CTagObj& aTag)
    {
        CTag* nonConstTag = const_cast<CTag*>(aTag);
        return aEngine->newQObject(nonConstTag);
    };

    static void fromScriptValue(const QScriptValue& aScriptValue, CTagObj& aTag) 
    {
        QObject* qobj = aScriptValue.toQObject();
        aTag = static_cast<CTag*>(qobj);
    };

    CTag &operator= (const QVariant &aValue)  {
        setValue(aValue);
        return *this;
    }

public slots:
    QString name();
    void setValue(const QVariant& aValue);
    QVariant value();      

private:
    QString mName;
    CRow  *mpRow;

    friend class CRow;
};

Q_SCRIPT_DECLARE_QMETAOBJECT(CReport,  QObject*)
Q_SCRIPT_DECLARE_QMETAOBJECT(CSection, QObject*)
Q_SCRIPT_DECLARE_QMETAOBJECT(CRow,     QObject*)
Q_SCRIPT_DECLARE_QMETAOBJECT(CTag,     QObject*)

Q_DECLARE_METATYPE(CTag*)
Q_DECLARE_METATYPE(CRow*)
Q_DECLARE_METATYPE(CSection*)

#endif // _CREPORT_H_
