/****************************************************************************
 **
 ** Copyright (C) 2008-2009 Grigory A. Mozhaev.  All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** AUTHOR: Grigory A. Mozhaev <zcrendel@mail.ru>
 **
 ****************************************************************************/
//! -*- coding: UTF-8 -*-
#define SOURCE_CODING "UTF-8"

#include <QMessageBox>
#include <CReportFactory>

CReportFactory *mpFactory = NULL;

void CReportFactory::init(const QString &aPluginsPath) {
  if (mpFactory == NULL) {
    mpFactory = new CReportFactory(aPluginsPath);
  }
}


void CReportFactory::finalize() {
  if (mpFactory) {
    delete mpFactory;
  }
}

CReportFactory *CReportFactory::factory() {
  return mpFactory;
}

void CReportFactory::addTemplate(const QString &aName, 
                                 const QString &aPath) {
    Q_ASSERT(mpFactory!=0);
    
    qDebug() << "Add new Template: " << aName << aPath;
    mpFactory->mTemplates.insert(aName,aPath);
}

void CReportFactory::removeTemplate(const QString &aName) {
    Q_ASSERT(mpFactory!=0);
    
    mpFactory->mTemplates.remove(aName);
}

QStringList CReportFactory::templates() {
    Q_ASSERT(mpFactory!=0);
    
    return mpFactory->mTemplates.keys();
}

QString CReportFactory::templatePath(const QString &aName) {
    Q_ASSERT(mpFactory!=0);

    if (mpFactory->mTemplates.contains(aName)) {
        return mpFactory->mTemplates[aName];
    }
    return QString::null;
}

void CReportFactory::clearTemplates() {
    Q_ASSERT(mpFactory!=0);

    mpFactory->mTemplates.clear();
}

CReportFactory::CReportFactory() {
    mPluginsPath = qApp->applicationDirPath();
    parseDir();
}

CReportFactory::CReportFactory(const QString &aPluginsPath) {
    mPluginsPath = aPluginsPath;
    parseDir();
}

IReport * CReportFactory::instance(const QString &aPluginName) {
    if (mFactoryStorage.contains(aPluginName)) {
        IReportPlugin * vInterface = mFactoryStorage[aPluginName];
        if (vInterface) {
            return vInterface->create();
        }
    }
    QMessageBox::warning(NULL, tr("Error occured during creating plugin"),
                               tr("Plugin name:  %1").arg(aPluginName));
    return NULL;
}

void CReportFactory::parseDir() {
    QObjectList vInstances = QPluginLoader::staticInstances();
    for (int i = 0; i < vInstances.count(); ++i) {
        IReportPlugin *vFactory = qobject_cast<IReportPlugin *>(vInstances.at(i));
        if (vFactory)
            mFactoryStorage.insert(vFactory->name(), vFactory);
    }
    parseAndLoad(QString("%1/plugins").arg(mPluginsPath));
}

void CReportFactory::parseAndLoad(const QString &aPluginsPath) {
    if (!aPluginsPath.isEmpty()) {
        QStringList vPluginsList = QDir(aPluginsPath).entryList(QDir::Files);
        qDebug() << "PLUGINS " << aPluginsPath;
        foreach (QString vPluginName, vPluginsList) {
            mLoader.setFileName(QDir::toNativeSeparators(aPluginsPath+"/"+vPluginName));
            if (mLoader.load()) {
                QObject * obj = mLoader.instance();
                IReportPlugin * vFactory = qobject_cast<IReportPlugin *>(obj);
                if (vFactory) {
                    mFactoryStorage.insert(vFactory->name(), vFactory);
                } else {
                  qDebug() << "problem to create instance of " << vPluginName << " Error cast.";
                }
            } else {
                qDebug() << "Error loading plugin: " << mLoader.errorString();
            }
        }
    }
}

QStringList CReportFactory::loadedPlugins() {
    return mFactoryStorage.keys();
}
