/****************************************************************************
 **
 ** Copyright (C) 2008-2009 Grigory A. Mozhaev.  All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** AUTHOR: Grigory A. Mozhaev <zcrendel@mail.ru>
 **
 ****************************************************************************/
#ifndef _CREPORTFACTORY_H_
#define _CREPORTFACTORY_H_

#include <IReport>
#include <IReportPlugin>

class CReportFactory : public QObject {
Q_OBJECT
public:
    CReportFactory();
    CReportFactory(const QString &aPluginsPath);

    IReport *instance(const QString &aPluginName);
    QStringList loadedPlugins();
    
    static void addTemplate(const QString &aName, const QString &aPath);
    static void removeTemplate(const QString &aName);
    static QStringList templates();
    static QString templatePath(const QString &aName);
    static void clearTemplates();

    static void generateReport();
    static void init(const QString &aPluginsPath);
    static CReportFactory * factory();
    static void finalize();

private:
    void parseDir();
    void parseAndLoad(const QString &aPluginsPath);
    QPluginLoader mLoader;

    // хранилища загруженных (зарегистрированных) в фабрике плагинов
    QStringList mPluginsList;
    QMap<QString, IReportPlugin *> mFactoryStorage;

    // текущий путь к директории, содержащей плагины
    QString mPluginsPath;

    QMap<QString,QString> mTemplates;
};

#endif // _CREPORTFACTORY_H_
