/****************************************************************************
 **
 ** Copyright (C) 2008-2009 Grigory A. Mozhaev.  All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** AUTHOR: Grigory A. Mozhaev <zcrendel@mail.ru>
 **
 ****************************************************************************/
#define SOURCE_CODING "UTF-8"

#include <QtCore>
#include <QEvent>
#include <QtXml>

#include <CFsUtils>
#include "codfreport.h"


#define gBeginSingleSeparator QLatin1String("<:")
#define gEndSingleSeparator   QLatin1String(":>")

#define gBeginMultipleSeparator QLatin1String("\\[:")
#define gEndMultipleSeparator   QLatin1String(":\\]")


COdfReport::COdfReport(QObject *apParent) {
    qWarning("COdfReport()");
}

COdfReport::~COdfReport() {
    qWarning("~COdfReport()");
}


void COdfReport::insertTagsValues(QDomNode &aNode,
                                const QString &aTagName,
                                const QString &aTagValue) {
  if (!aTagValue.isNull()) {
      aNode.setNodeValue(aNode.nodeValue().replace(
         QString("%1%2%3").arg(gBeginSingleSeparator)
                          .arg(aTagName)
                          .arg(gEndSingleSeparator),
         aTagValue));
  }
}


void COdfReport::addSimpleTag(QDomNode &aNode,
                            const QString& aTagName,
                            const QString& aTagValue) {
    QDomNode vNode = aNode.lastChild();

    QRegExp vRegExp;
    vRegExp.setMinimal(true);

    while (!vNode.isNull()) {
        vRegExp.setPattern(QString("%1.*%2")
                           .arg(gBeginMultipleSeparator)
                           .arg(gEndMultipleSeparator));


        bool vRetVal = getNodeTags(vNode, aTagName, vRegExp);
        if(!vRetVal) {
            vRegExp.setPattern(QString("%1.*%2")
                               .arg(gBeginSingleSeparator)
                               .arg(gEndSingleSeparator));

            vRetVal = getNodeTags(vNode, aTagName, vRegExp);
            if (vRetVal)    {
                insertTagsValues(vNode, aTagName, aTagValue);
            } else  {
                addSimpleTag(vNode, aTagName, aTagValue);
            }
        }
        vNode = vNode.previousSibling();
    }
}

void COdfReport::insertRowValues(QDomNode &aNode, 
                               uint aId,
                               const QString& aSecName,
                               uint aRow) {
  QRegExp vMultiRegExp;
  vMultiRegExp.setPattern(QString("%1.*%2")
                          .arg(gBeginMultipleSeparator)
                          .arg(gEndMultipleSeparator));
  vMultiRegExp.setMinimal(true);

  QDomNode vNode = aNode;
  while (!vNode.parentNode().isNull()) {
      vNode = vNode.parentNode();
      if (vNode.nodeName()=="table:table-row") {
          vNode.parentNode().insertAfter(vNode.cloneNode(true), vNode);
          clearTags(vNode, vMultiRegExp);

          QSqlQueryModel vRows;
          vRows.setQuery(QString("SELECT * FROM rows WHERE section=%1 AND row=%2")
                         .arg(aId).arg(aRow));
          bool vFlag = vRows.query().first();

          if (!vFlag) {
              qDebug() << "no records!" 
                       << aId << aSecName << aRow;
          }

          while (vRows.query().isValid()) {
              QString vTagName  = vRows.query().record().value("tag").toString();
              QString vTagValue = vRows.query().record().value("value").toString();
              //qDebug() << "replace tag in row: " << aId << aSecName << aRow <<  vTagName << vTagValue;
              addSimpleTag(vNode, vTagName, vTagValue);
              vRows.query().next();
          } 
      }
  }
}

void COdfReport::addSectionRow(QDomNode &aNode, 
                            uint aId,
                            const QString& aSecName,
                            uint aRow) {
    QDomNode vNode = aNode.lastChild();

    QRegExp vRegExp;
    vRegExp.setMinimal(true);

    while (!vNode.isNull()) {
        vRegExp.setPattern(QString("%1.*%2")
                           .arg(gBeginMultipleSeparator)
                           .arg(gEndMultipleSeparator));

        bool vRetVal = getNodeTags(vNode, aSecName, vRegExp);
        if(vRetVal) {
            //qDebug() << "found row section" << aSecName;
            insertRowValues(vNode, aId, aSecName, aRow);
        } else {
            addSectionRow(vNode, aId, aSecName, aRow);
        }
        vNode = vNode.previousSibling();
    }
}

int COdfReport::generate(IReportData *aData) {
    Q_ASSERT(aData!=0);
    
    qWarning("ibReport::generate()");
    mpData = aData;

    mTemplatePath = mpData->templatePath(mpData->options()["Template"].toString());
    mTmpDir = mpData->workDir();
    qDebug() << "WORKDIR: " << mTmpDir << " TEMPLATE: " << mTemplatePath;

    if (prepare(mTemplatePath)!=0) {
        qDebug() << "error during prepare report";
        return 1;
    }

    /* here must be replacing of tags */
    QSqlQueryModel vSecs;
    vSecs.setQuery(QString("SELECT id, name FROM sections"));
    vSecs.query().first();
    while (vSecs.query().isValid()) {
        uint vId      = vSecs.query().record().value("id").toUInt();
        QString vName = vSecs.query().record().value("name").toString();

        QSqlQueryModel vRows;
        if (vName.isNull()) {
            qWarning("found main section! (ID = %u)", vId);
            vRows.setQuery(QString("SELECT * FROM rows WHERE section=%1 AND row=1").arg(vId));
            vRows.query().first();
            while (vRows.query().isValid()) {
                QString vTagName  = vRows.query().record().value("tag").toString();
                QString vTagValue = vRows.query().record().value("value").toString();

                QDomNode vNode = mContent.lastChild();
                while (!vNode.isNull()) {
                    addSimpleTag(vNode, vTagName, vTagValue);
                    vNode = vNode.previousSibling();
                }
                vRows.query().next();
            }
        } else {
            vRows.setQuery(QString("SELECT DISTINCT row FROM rows WHERE section=%1 ORDER BY row").arg(vId));
            vRows.query().first();
            uint vPrevRow = 0;

            while (vRows.query().isValid()) {
                uint vRow = vRows.query().record().value("row").toUInt();
            
                for (int i=1; i < (vRow - vPrevRow + 1); i++) {
                    QDomNode vNode = mContent.lastChild();
                    while (!vNode.isNull()) {
                        addSectionRow(vNode, vId, vName, vPrevRow + i);
                        vNode = vNode.previousSibling();
                    }
                }
                vPrevRow = vRow;
                vRows.query().next();
            }
        }

        vSecs.query().next();
    }

    QString vReport = QDir::toNativeSeparators(COdfReport::mTmpDir+QLatin1String("/report.odt"));

    if (QFile::exists(vReport))
        QFile::remove(vReport);

    if (saveAs(vReport) != 0) {
        qDebug() << "save report failed";
        return 1;
    }

    QProcess vProcess;
    QString  vProgram;

    if (QFile::exists("/usr/bin/ooffice")) {
        vProgram = QLatin1String("/usr/bin/ooffice");
    } else {
        vProgram = QLatin1String("/usr/bin/ooffice2");
    }

    QStringList vArguments;
    vArguments << QDir::convertSeparators(vReport);

    vProcess.start(vProgram, vArguments);

    if (!vProcess.waitForStarted()) {
        QMessageBox::warning(0, tr("Warning"),   tr("Unable to start OpenOffice"), 
                             QMessageBox::Ok, QMessageBox::NoButton);
        return 1;
    }

    while ((!vProcess.waitForFinished(1000)) && 
           (vProcess.state()==QProcess::Running)) {
        qApp->processEvents();
    }

    return 0;
}

// private members 
// функции и члены-данные для работы с ODF

int COdfReport::prepare(const QString &aTemplateFile) {
    // unzip document to workdir
    mWorkDir = QDir::toNativeSeparators(QString("%1/doc-%2/")
                                        .arg(mTmpDir)
                                        .arg(QDateTime::currentDateTime()
                                             .toString("MMddhhmmsszzz")));

    int vCode = CFsUtils::extractZipFile(aTemplateFile, mWorkDir);
    if (vCode == CFsUtils::ECODE_SUCCESS) {
        // load content and style structures
        QFile vContent (QDir::convertSeparators(QString("%1/%2")
                                                .arg(mWorkDir).arg("/content.xml"))); 
        mContent.setContent(&vContent, false);

        return 0;
    } else {
        qDebug() << "Error: CFsUtils::" << vCode << 
            "(" << CFsUtils::mErrorMessage << ")";
    }

    return 1;
}

int COdfReport::saveAs(const QString &aFileName) {
    finishReport();

    QFile vContent(QDir::convertSeparators(mWorkDir+"/content.xml") );
    if (!vContent.open(QIODevice::WriteOnly)) {
        return false;
    }
    QTextStream vContentStream(&vContent);
    mContent.save(vContentStream,2);
    vContent.close();
    
    lineBreakRemove(vContent.fileName());

    CFsUtils::compressZipFile(aFileName, mWorkDir);
    
    QFile vReport(aFileName);    
    vReport.setPermissions(QFile::ReadOwner | QFile::ReadGroup | QFile::ReadOther);
        
    return 0;
}

bool COdfReport::lineBreakRemove(const QString &aFileName) {
    qWarning("COdfReport::lineBreakRemove()");
    QFile   vFile(aFileName);
    QString vContent("");

    if (vFile.open( QIODevice::ReadOnly )) {
        QTextStream vStream(&vFile);
        vStream.setCodec("UTF-8");

        while (!vStream.atEnd())
            vContent += vStream.readLine();

        vFile.close();
    } else {
        qWarning("COdfReport::lineBreakRemove() : Cannot read %s file",QFile::encodeName(aFileName).data());
        return false;
    }
    
    if (vFile.open(QIODevice::WriteOnly)) {
        vFile.write(vContent.toUtf8(), vContent.toUtf8().size());
        vFile.close();
    }
    else {
        qWarning("COdfReport::lineBreakRemove() : Cannot write %s file", QFile::encodeName(aFileName).data());
        return false;
    }

    return true;
}

bool COdfReport::getNodeTags(QDomNode &aNode, const QString &aTagName,
                           QRegExp &aRegExp) {
    //qWarning("COdfReport::getNodeTags()");
    if(aNode.isText()) {
        QString vText  = aNode.nodeValue();
        int     vPos   = aRegExp.indexIn(vText,0);

        while(vPos != -1) {
            if(aTagName == vText.mid(vPos+2, aRegExp.matchedLength()-4))    {
                return true;
            }

            vPos += aRegExp.matchedLength();
            vPos  = aRegExp.indexIn(vText, vPos);
        }
    }

    return false;
}

void COdfReport::finishReport() {
  qWarning("COdfReport::finishReport()");

  //clear tags in content
  QRegExp vSingleRegExp;
  vSingleRegExp.setPattern(QString("%1.*%2")
                           .arg(gBeginSingleSeparator)
                           .arg(gEndSingleSeparator));
  vSingleRegExp.setMinimal(true);
  
  QDomNode vNode = mContent.lastChild();
  while (!vNode.isNull()) {
      clearTags(vNode, vSingleRegExp);
      vNode = vNode.previousSibling();
  }

  vNode = mContent.lastChild();
  while (!vNode.isNull()) {
      clearRows(vNode);
      vNode = vNode.previousSibling();
  }
}

void COdfReport::clearTags(QDomNode &aNode,  QRegExp  &aRegExp) {
    //qWarning("COdfReport::clearTags()");
  if (aNode.isNull()) return;

  QDomNode vNode = aNode.lastChild();
  while (!vNode.isNull())   {
      if(vNode.isText()) {
          QString vText = vNode.nodeValue();

          do {
              int vPos = aRegExp.indexIn(vText,0);
              if (vPos >= 0)
                  vText = vText.remove(aRegExp);
              else
                  break;
          }
          while (true);
          
          vNode.setNodeValue(vText);
      } else    {
          clearTags(vNode, aRegExp);
      }
      vNode = vNode.previousSibling();
  }
}

void COdfReport::clearRows(QDomNode &aNode) {
    //qWarning("COdfReport::clearRows()");

  QDomNode vNode = aNode.lastChild();

  while (!vNode.isNull()) { 
      if (vNode.isText()) {
          QString vText = vNode.nodeValue();
          QRegExp vRegExp;
          vRegExp.setPattern(QString("%1.*%2")
                             .arg(gBeginMultipleSeparator)
                             .arg(gEndMultipleSeparator));
          vRegExp.setMinimal(true);
          
          int vPos = vRegExp.indexIn(vText, 0);
          if (vPos >= 0) {
              QDomNode vTmpNode = vNode;
              while(!vTmpNode.parentNode().isNull()) {
                  vTmpNode = vTmpNode.parentNode();
                  if (vTmpNode.nodeName() == "table:table-row") {
                      vTmpNode.parentNode().removeChild(vTmpNode);
                      break;
                  }
              }
          }
      } else {
          clearRows(vNode);
      }
      
      vNode = vNode.previousSibling();
  }
}
