/****************************************************************************
 **
 ** Copyright (C) 2008-2009 Grigory A. Mozhaev.  All rights reserved.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
 ** http://www.trolltech.com/products/qt/opensource.html
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** AUTHOR: Grigory A. Mozhaev <zcrendel@mail.ru>
 **
 ****************************************************************************/
#ifndef _CODFREPORT_H_
#define _CODFREPORT_H_

#include <QtCore>
#include <QtGui>
#include <QtXml>
#include <QtSql>
#include <IReport>

class COdfReport : public QObject, public IReport {
Q_OBJECT

public:
    COdfReport(QObject *apParent = 0);
    ~COdfReport();
  
    int generate(IReportData *aData);

private:
    // функции и члены-данные для работы с ODF
    int   prepare(const QString &aTemplateFile);
    int   saveAs(const QString &aFileName);

    bool   lineBreakRemove(const QString &aFileName);
    
    void   searchTags(QDomNode &aNode, const QString &aTagName);
    bool   getNodeTags(QDomNode &aNode, const QString &aTagName, QRegExp &aRegExp);

    void   finishReport();
    void   clearTags(QDomNode &aNode,  QRegExp  &aRegExp);
    void   clearRows(QDomNode &aNode);

    QString tagValue(const QString &aTagName);

    void   insertRowValues(QDomNode &aNode, 
                           uint aId,
                           const QString& aSecName,
                           uint aRow);

    void insertTagsValues(QDomNode &aNode, const QString &aTagName,
                          const QString &aTagValue);

    void addSimpleTag(QDomNode &aNode,
                      const QString& aTagName,
                      const QString& aTagValue);

    void addSectionRow(QDomNode &aNode,
                       uint aId,
                       const QString& aSecName,
                       uint aRow);

    // Содержимое Odf документа
    QDomDocument           mOdfContent;
    // Стили Odf документа (содержат колонтитулы)
    QDomDocument           mOdfStyles;
    // Рабочий каталог с содержимым документа
    QString mWorkDir;

    // Содержимое ODF документа
    QDomDocument           mContent;

    IReportData *mpData;

    QString mTmpDir;
    QString mTemplatePath;
};

#endif // _CODFREPORT_H_
